@@ -59,6 +59,7 @@
 #include "utils/pool_signal.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
+#include "pool_config_variables.h"
 
 char *copy_table = NULL;  /* copy table name */
 char *copy_schema = NULL;  /* copy table name */
@@ -89,6 +90,74 @@ static POOL_STATUS parse_before_bind(POOL_CONNECTION *frontend,
 static int* find_victim_nodes(int *ntuples, int nmembers, int master_node, int *number_of_nodes);
 static POOL_STATUS close_standby_transactions(POOL_CONNECTION *frontend,
 											  POOL_CONNECTION_POOL *backend);
+
+static char *
+flatten_set_variable_args(const char *name, List *args);
+static bool
+process_pg_terminate_backend_func(POOL_QUERY_CONTEXT *query_context);
+
+/*
+ * This is the workhorse of processing the pg_terminate_backend function to
+ * make sure that the use of function should not trigger the backend node failover.
+ *
+ * The function searches for the pg_terminate_backend() function call in the
+ * query parse tree and if the search comes out to be successful,
+ * the next step is to locate the pgpool-II child process and a backend node
+ * of that connection whose PID is specified in pg_terminate_backend argument.
+ *
+ * Once the connection is identified, we set the swallow_termination flag of
+ * that connection (in shared memory) and also sets the query destination to
+ * the same backend that hosts the connection.
+ *
+ * The function returns true on success, i.e. when the query contains the
+ * pg_terminate_backend call and that call refers to the backend
+ * connection that belongs to pgpool-II.
+ *
+ * Note:  Since upon successful return this function has already
+ * set the destination backend node for the current query,
+ * so for that case pool_where_to_send() should not be called.
+ *
+ */
+static bool process_pg_terminate_backend_func(POOL_QUERY_CONTEXT *query_context)
+{
+	/*
+	 * locate pg_terminate_backend and get the pid argument, if pg_terminate_backend
+	 * is present in the query
+	 */
+	int backend_pid = pool_get_terminate_backend_pid(query_context->parse_tree);
+	if (backend_pid > 0)
+	{
+		int backend_node = 0;
+		ConnectionInfo* conn = pool_coninfo_backend_pid(backend_pid, &backend_node);
+		if(conn == NULL)
+		{
+			ereport(LOG,
+				(errmsg("found the pg_terminate_backend request for backend pid:%d, but the backend connection does not belong to pgpool-II",backend_pid)));
+			/* we are not able to find the backend connection with the pid
+			 * so there is not much left for us to do here
+			 */
+			return false;
+		}
+		ereport(LOG,
+			(errmsg("found the pg_terminate_backend request for backend pid:%d on backend node:%d",backend_pid,backend_node),
+				 errdetail("setting the connection flag")));
+
+		if (pool_is_my_coninfo(conn)){
+			ereport(LOG,
+				(errmsg("pg_terminate_backend refer to the current child process connection"),
+					 errdetail("setting the connection flag not required")));
+		}
+		else{
+			pool_set_connection_will_be_terminated(conn);
+		}
+		/* It was the pg_terminate_backend call so send the query to appropriate backend */
+		query_context->pg_terminate_backend_conn = conn;
+		pool_force_query_node_to_backend(query_context, backend_node);
+		return true;
+	}
+	return false;
+}
+
 /*
  * Process Query('Q') message
  * Query messages include an SQL string.
@@ -258,6 +327,43 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		 */
 		check_copy_from_stdin(node);
 
+		if (IsA(node, PgpoolVariableShowStmt))
+		{
+			VariableShowStmt *vnode = (VariableShowStmt *)node;
+
+			report_config_variable(frontend, backend, vnode->name);
+
+			pool_ps_idle_display(backend);
+			pool_query_context_destroy(query_context);
+			pool_set_skip_reading_from_backends();
+			return POOL_CONTINUE;
+		}
+
+		if (IsA(node, PgpoolVariableSetStmt))
+		{
+			VariableSetStmt *vnode = (VariableSetStmt *)node;
+			const char *value = NULL;
+			if (vnode->kind == VAR_SET_VALUE)
+			{
+				value = flatten_set_variable_args("name", vnode->args);
+				printf("%s\n", value);
+			}
+			if (vnode->kind == VAR_RESET_ALL)
+			{
+				printf("reset all\n");
+				reset_all_variables(frontend, backend);
+			}
+			else
+			{
+				set_config_option_for_session(frontend, backend, vnode->name, value);
+			}
+
+			pool_ps_idle_display(backend);
+			pool_query_context_destroy(query_context);
+			pool_set_skip_reading_from_backends();
+			return POOL_CONTINUE;
+		}
+
 		/* status reporting? */
 		if (IsA(node, VariableShowStmt))
 		{
@@ -368,11 +474,18 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		}
 
 		/*
-		 * Decide where to send query
+		 * pg_terminate function needs special handling, process it if the query
+		 * contains one, otherwise use pool_where_to_send() to decide destination
+		 * backend node for the query
 		 */
-		pool_where_to_send(query_context, query_context->original_query,
-						   query_context->parse_tree);
-
+		if (process_pg_terminate_backend_func(query_context) == false)
+		{
+			/*
+			 * Decide where to send query
+			 */
+			pool_where_to_send(query_context, query_context->original_query,
+							   query_context->parse_tree);
+		}
 		/*
 		 * if this is DROP DATABASE command, send USR1 signal to parent and
 		 * ask it to close all idle connections.
@@ -387,7 +500,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 
 			ereport(DEBUG1,
 					(errmsg("Query: sending SIGUSR1 signal to parent")));
-			register_node_operation_request(CLOSE_IDLE_REQUEST, NULL, 0);
+			register_node_operation_request(CLOSE_IDLE_REQUEST, NULL, 0, false, 0);
 
 			/* we need to loop over here since we will get USR1 signal while sleeping */
 			while (stime > 0)
@@ -1519,7 +1632,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 
 				free_string(msg);
 
-				degenerate_backend_set(victim_nodes, number_of_nodes);
+				degenerate_backend_set(victim_nodes, number_of_nodes, true, 0);
 				child_exit(POOL_EXIT_AND_RESTART);
 			}
 			else
@@ -2518,14 +2631,16 @@ POOL_STATUS ProcessBackendResponse(POOL_CONNECTION *frontend,
 
 			case 's':	/* PortalSuspended */
 				status = SimpleForwardToFrontend(kind, frontend, backend);
-				if ((REPLICATION || RAW_MODE) && pool_is_doing_extended_query_message())
+				if (pool_is_doing_extended_query_message())
 					pool_unset_query_in_progress();
 				break;
 
 			default:
 				status = SimpleForwardToFrontend(kind, frontend, backend);
+#ifdef NOT_USED
 				if (pool_flush(frontend))
 					return POOL_END;
+#endif
 				break;
 		}
 	}
@@ -3162,6 +3277,7 @@ static int* find_victim_nodes(int *ntuples, int nmembers, int master_node, int *
 	return victim_nodes;
 }
 
+
 /*
  * flatten_set_variable_args
  *		Given a parsenode List as emitted by the grammar for SET,
